import json
import os
import sys

sys.path.insert(0,os.getcwd())

import jwt 

from bottle import Bottle, request, response
from BottleSessions import BottleSessions
from BottleOIDC import BottleOIDC

from config import oidc_config

DEBUG = os.environ.get('DEBUG', False)

app = Bottle()
BottleSessions(app, session_expire=4*60*60, session_backing={
    'cache_type': 'FileSystem', 
    'cache_dir': './.cache_dir'
})

auth = BottleOIDC(app, config=oidc_config)

#
# Authentication: require_login decorator
#
@app.route(['/login','/logon'])
@auth.require_login
def login():
    """ Login """

    return {'username': request.session['username']}

#
# logout 
#
@app.route(['/logout','/logoff'])
def bye():
    """ Logout """

    if auth.is_authenticated:
        return auth.initiate_logout(next=request.url)
    else:
        return 'ok'

#
# Authorization: require_user decorator
#
@app.route('/bob')
@auth.require_user(['bob','r.r.kras'])
def bob():
    """ Only bob """
    return 'hi bob'

#
# Authorization: require_attribute decorator
#
@app.route('/sa')
@auth.require_attribute('groups', ['netadmin', 'sysadmin'])
def sa():
    return 'hi sa'


if DEBUG:
    # DEBUG helpers

    print(auth.scopes_supported)
    @app.route('/.sess')
    def status():
        """ Return entire session """
        response.set_header('Content-Type', 'text/plain')
        return json.dumps(request.session, indent=4)


    @app.route('/.exp')
    def expire():
        """ Hasten token expiration by 1 hr. """
        request.session[auth.token_name]['exp'] -= 3600
        request.session.session_modified = True
        return 'ok'


    def decode_jwt(token):
        """ decode and return token response. """

        payload = jwt.decode(token, options={'verify_signature': False})
        response.set_header('Content-Type', 'text/plain')
        return json.dumps(payload, indent=4)


    @app.route(['/<pat>/.id','/.id'])
    def id_tok(pat=auth.token_name):
        """ decode id token """

        return decode_jwt(request.session[pat]['id_token'])


    @app.route(['/<pat>/.access','/.access'])
    def ac_tok(pat=auth.token_name):
        """ decode access token """

        return decode_jwt(request.session[pat]['access_token'])


    @app.route(['/.meh','/.meh/'])
    def meh_token():
        """ Request new tokens with new scope. """
        scp = request.query.get('scp')
        tokens = auth.get_access_token('meh', scope=scp)
        return {'tokens': tokens}


if __name__ == '__main__':
    app.run(port=8000, debug=DEBUG, reloader=DEBUG)




# Example configuration template
oidc_config = {
    "client_id": "IDP Provided Client ID",
    "client_secret": "IDP Provided Secret",
    "discovery_url": "https://login.microsoftonline.com/<tenentid>/V2.0/.well-known/openid-configuration",
    "client_scope": ["openid", "email", "profile", ],
    "user_attr": 'email'
}


import os
import sys 

DEBUG = os.environ.get('DEBUG', False)

from bottle import request, response

def set_no_cache_headers():
    """
    Set various "no cache" headers for this response
    """

    # MDN recommended for various browsers
    response.add_header('Cache-Control', 'no-cache')
    response.add_header('Cache-Control', 'must-revalidate')
    response.add_header('Pragma', 'no-cache')
    response.add_header('Expires', 'Sun, 25 Jul 2021 15:42:14 GMT')


def url_for(name, _external=False):
    """
    mimic Flask url_for
    """
    path_part = request.app.get_url(name)
    if not _external:
        return path_part

    # External URL
    host_part = request.urlparts.scheme + '://' + request.urlparts.netloc
    url = host_part + path_part

    return url


def redirect(url, status=302):
    """ Update response for redirect """

    response.status = status
    response.add_header('Location', url)
    set_no_cache_headers()
    return None

#
# Assure consistent error responses
#
def response_error(status=401, body='', hdrs=None):
        response.status = status
        set_no_cache_headers()
        response.headers.update(hdrs or {})
        return body

def UnauthorizedError(body='Unauthorized'):
    return response_error(status=401, body=body)

def BadRequestError(body='Bad Request'):
    return response_error(status=400, body=body)

def AppError(body='App error'):
    return response_error(status=400, body=body)

def ConflictError(body='Conflict Error'):
    return response_error(status=400, body=body)

def ForbiddenError(body='Forbidden'):
    return response_error(status=403, body=body)

def NotFoundError(body='Not Found'):
    return response_error(status=404, body=body)

class _Log:
    """
    Mock logger - mocks logging with no bells and whistles
    """
    level = 'info'

    def info(self, *args, **kwargs):
        if self.level in ['info']:
            print('INFO -', *args, **kwargs, file=sys.stderr)

    def warn(self, *args, **kwargs):
        print('WARN -', *args, **kwargs, file=sys.stderr)

    def debug(self, *args, **kwargs):
        if DEBUG:
            print('DEBUG -', *args, **kwargs, file=sys.stderr)

    def error(self, *args, **kwargs):
        print('ERROR -', *args, **kwargs, file=sys.stderr)

